@@ -1,12 +1,16 @@
 /* See md5.c for explanation and copyright information.  */
 
 /*
- * $Id: md5.h 1973 2007-09-21 16:10:16Z peter $
+ * $Id: md5.h 2101 2008-05-23 06:46:51Z peter $
  */
 
 #ifndef YASM_MD5_H
 #define YASM_MD5_H
 
+#ifndef YASM_LIB_DECL
+#define YASM_LIB_DECL
+#endif
+
 /* Unlike previous versions of this code, uint32 need not be exactly
    32 bits, merely 32 bits or more.  Choosing a data type which is 32
    bits instead of 64 is not important; speed is considerably more
@@ -19,10 +23,14 @@ typedef struct yasm_md5_context {
         unsigned char in[64];
 } yasm_md5_context;
 
+YASM_LIB_DECL
 void yasm_md5_init(yasm_md5_context *context);
+YASM_LIB_DECL
 void yasm_md5_update(yasm_md5_context *context, unsigned char const *buf,
                      unsigned long len);
+YASM_LIB_DECL
 void yasm_md5_final(unsigned char digest[16], yasm_md5_context *context);
+YASM_LIB_DECL
 void yasm_md5_transform(unsigned long buf[4], const unsigned char in[64]);
 
 #endif /* !YASM_MD5_H */
