@@ -3,7 +3,7 @@
  * \brief YASM object format module interface.
  *
  * \rcs
- * $Id: objfmt.h 1973 2007-09-21 16:10:16Z peter $
+ * $Id: objfmt.h 2166 2009-01-02 08:33:21Z peter $
  * \endrcs
  *
  * \license
@@ -62,6 +62,9 @@ struct yasm_objfmt_module {
      */
     const unsigned char default_x86_mode_bits;
 
+    /** If @ signs should be legal in identifiers. */
+    const unsigned char id_at_ok;
+
     /** NULL-terminated list of debug format (yasm_dbgfmt) keywords that are
      * valid to use with this object format.  The null debug format
      * (null_dbgfmt, "null") should always be in this list so it's possible to
@@ -77,6 +80,9 @@ struct yasm_objfmt_module {
     /** NULL-terminated list of directives.  NULL if none. */
     /*@null@*/ const yasm_directive *directives;
 
+    /** NULL-terminated list of standard macro lookups.  NULL if none. */
+    const yasm_stdmac *stdmacs;
+
     /** Create object format.
      * Module-level implementation of yasm_objfmt_create().
      * Call yasm_objfmt_create() instead of calling this function.
@@ -109,6 +115,13 @@ struct yasm_objfmt_module {
         (*section_switch)(yasm_object *object, yasm_valparamhead *valparams,
                           /*@null@*/ yasm_valparamhead *objext_valparams,
                           unsigned long line);
+
+    /** Module-level implementation of yasm_objfmt_get_special_sym().
+     * Call yasm_objfmt_get_special_sym() instead of calling this function.
+     */
+    /*@observer@*/ /*@null@*/ yasm_symrec *
+        (*get_special_sym)(yasm_object *object, const char *name,
+                           const char *parser);
 };
 
 /** Create object format.
@@ -156,6 +169,16 @@ yasm_section *yasm_objfmt_add_default_section(yasm_object *object);
     (yasm_object *object, yasm_valparamhead *valparams,
      /*@null@*/ yasm_valparamhead *objext_valparams, unsigned long line);
 
+/** Get a special symbol.  Special symbols are generally used to generate
+ * special relocation types via the WRT mechanism.
+ * \param object        object
+ * \param name          symbol name (not including any parser-specific prefix)
+ * \param parser        parser keyword
+ * \return NULL if unrecognized, otherwise special symbol.
+ */
+/*@observer@*/ /*@null@*/ yasm_symrec *yasm_objfmt_get_special_sym
+    (yasm_object *object, const char *name, const char *parser);
+
 #ifndef YASM_DOXYGEN
 
 /* Inline macro implementations for objfmt functions */
@@ -173,6 +196,9 @@ yasm_section *yasm_objfmt_add_default_section(yasm_object *object);
 #define yasm_objfmt_add_default_section(object) \
     ((yasm_objfmt_base *)((object)->objfmt))->module->add_default_section \
         (object)
+#define yasm_objfmt_get_special_sym(object, name, parser) \
+    ((yasm_objfmt_base *)((object)->objfmt))->module->get_special_sym \
+        (object, name, parser)
 
 #endif
 
