@@ -1,5 +1,5 @@
 /* Modified for use with yasm by Peter Johnson.
- * $Id: phash.h 1973 2007-09-21 16:10:16Z peter $
+ * $Id: phash.h 2101 2008-05-23 06:46:51Z peter $
  */
 /*
 ------------------------------------------------------------------------------
@@ -10,6 +10,12 @@ Source is http://burtleburtle.net/bob/c/lookupa.h
 ------------------------------------------------------------------------------
 */
 
+#ifndef YASM_LIB_DECL
+#define YASM_LIB_DECL
+#endif
+
+YASM_LIB_DECL
 unsigned long phash_lookup(const char *k, size_t length,
                            unsigned long level);
+YASM_LIB_DECL
 void phash_checksum(const char *k, size_t length, unsigned long *state);
